@@ -6,6 +6,8 @@ import string
 import re            
 import subprocess
 import gettext
+import os.path
+import macros
 
 from gettext import gettext as _
 from xml.etree.ElementTree import ElementTree
@@ -13,10 +15,10 @@ from xml.etree.ElementTree import ElementTree
 class LocaleInfo(object):
     " class with handy functions to parse the locale information "
     
-    environment = "/etc/environment"
+    environments = ["/etc/default/locale", "/etc/environment"]
     def __init__(self, languagelist_file):
         # map language to human readable name, e.g.:
-        # "pt"->"Portugise", "de"->"German", "en"->"English"
+        # "pt"->"Portuguise", "de"->"German", "en"->"English"
         self._lang = {}
 
         # map country to human readable name, e.g.:
@@ -27,31 +29,44 @@ class LocaleInfo(object):
         # "pt_PT"->"pt_PT:pt:pt_BR:en_GB:en"
         self._languagelist = {}
         
-        # read lang file (we don't use iso_639_3 as it does crash there with:
-        # xml.parsers.expat.ExpatError: not well-formed (invalid token): line 52, column 11
+        # read lang file
         et = ElementTree(file="/usr/share/xml/iso-codes/iso_639.xml")
         it = et.getiterator('iso_639_entry')
         for elm in it:
             lang = elm.attrib["name"]
-            if elm.attrib.has_key("iso_639_1_code"):
+            if "iso_639_1_code" in elm.attrib:
                 code = elm.attrib["iso_639_1_code"]
             else:
                 code = elm.attrib["iso_639_2T_code"]
-            self._lang[code] = gettext.dgettext('iso_639', lang)
-            
+            if not code in self._lang:
+                self._lang[code] = lang
+        # Hack for Chinese langpack split
+        # Translators: please translate 'Chinese (simplified)' and 'Chinese (traditional)' so that they appear next to each other when sorted alphabetically.
+        self._lang['zh-hans'] = _("Chinese (simplified)")
+        # Translators: please translate 'Chinese (simplified)' and 'Chinese (traditional)' so that they appear next to each other when sorted alphabetically.
+        self._lang['zh-hant'] = _("Chinese (traditional)")
+        # end hack
+        et = ElementTree(file="/usr/share/xml/iso-codes/iso_639_3.xml")
+        it = et.getiterator('iso_639_3_entry')
+        for elm in it:
+            lang = elm.attrib["name"]
+            code = elm.attrib["id"]
+            if not code in self._lang:
+                self._lang[code] = lang
+        
         # read countries
         et = ElementTree(file="/usr/share/xml/iso-codes/iso_3166.xml")
         it = et.getiterator('iso_3166_entry')
         for elm in it:
-            if elm.attrib.has_key("common_name"):
+            if "common_name" in elm.attrib:
                 descr = elm.attrib["common_name"]
             else:
                 descr = elm.attrib["name"]
-            if elm.attrib.has_key("alpha_2_code"):
+            if "alpha_2_code" in elm.attrib:
                 code = elm.attrib["alpha_2_code"]
             else:
                 code = elm.attrib["alpha_3_code"]
-            self._country[code] = gettext.dgettext('iso_3166',descr)
+            self._country[code] = descr
             
         # read the languagelist
         for line in open(languagelist_file):
@@ -70,19 +85,19 @@ class LocaleInfo(object):
 
     def lang(self, code):
         """ map language code to language name """
-        if self._lang.has_key(code):
+        if code in self._lang:
             return self._lang[code]
         return ""
 
     def country(self, code):
         """ map country code to country name"""
-        if self._country.has_key(code):
+        if code in self._country:
             return self._country[code]
         return ""
 
     def generated_locales(self):
-        """ return a list of locales avaialble on the system
-            (runing locale -a) """
+        """ return a list of locales available on the system
+            (running locale -a) """
         locales = []
         p = subprocess.Popen(["locale", "-a"], stdout=subprocess.PIPE)
         for line in string.split(p.communicate()[0], "\n"):
@@ -92,28 +107,63 @@ class LocaleInfo(object):
             # we are only interessted in the locale, not the codec
             locale = string.split(tmp)[0]
             locale = string.split(locale,".")[0]
-            locale = string.split(locale,"@")[0]
+            #locale = string.split(locale,"@")[0]
             if not locale in locales:
                 locales.append(locale)
         #print locales
         return locales
 
+    def translate_language(self, lang):
+        "return translated language"
+        if lang in self._lang:
+            lang_name = gettext.dgettext('iso_639', self._lang[lang])
+            if lang_name == self._lang[lang]:
+                lang_name = gettext.dgettext('iso_639_3', self._lang[lang])
+            return lang_name
+        else:
+            return lang
+
+    def translate_locale(self, locale):
+        """
+        return translated language and country of the given
+        locale into the given locale, e.g. 
+        (Deutsch, Deutschland) for de_DE
+        """
+
+        macr = macros.LangpackMacros(locale)
+
+        #(lang, country) = string.split(locale, "_")
+        country = macr['CCODE']
+        current_language = None
+        if "LANGUAGE" in os.environ:
+            current_language = os.environ["LANGUAGE"]
+        os.environ["LANGUAGE"]=locale
+        lang_name = self.translate_language(macr['LCODE'])
+        country_name = gettext.dgettext('iso_3166', self._country[country])
+        if current_language:
+            os.environ["LANGUAGE"] = current_language
+        else:
+            del os.environ["LANGUAGE"]
+        return (lang_name, country_name)
+
     def translate(self, locale):
         """ get a locale code and output a human readable name """
+        macr = macros.LangpackMacros(locale)
         if "_" in locale:
-            (lang, country) = string.split(locale, "_")
+            #(lang, country) = string.split(locale, "_")
+            (lang_name, country_name) = self.translate_locale(locale)
             # get all locales for this language
-            l = filter(lambda k: k.startswith(lang+"_"), self.generated_locales())
+            l = filter(lambda k: k.startswith(macr['LCODE']), self.generated_locales())
             # only show region/country if we have more than one 
             if len(l) > 1:
-                mycountry = self.country(country)
+                mycountry = self.country(macr['CCODE'])
                 if mycountry:
-                    return "%s (%s)" % (_(self.lang(lang)), _(mycountry))
+                    return "%s (%s)" % (lang_name, country_name)
                 else:
-                    return "%s" % (_(self.lang(lang)))
+                    return lang_name
             else:
-                return _(self.lang(lang))
-        return _(self.lang(locale))
+                return lang_name
+        return self.translate_language(locale)
 
     def makeEnvString(self, code):
         """ input is a language code, output a string that can be put in
@@ -121,7 +171,7 @@ class LocaleInfo(object):
             E.g: en_DK -> en_DK:en
         """
         # first check if we got somethign from languagelist
-        if self._languagelist.has_key(code):
+        if code in self._languagelist:
             return self._languagelist[code]
         # if not, fall back to "dumb" behaviour
         if not "_" in code:
@@ -131,22 +181,24 @@ class LocaleInfo(object):
 
     def getDefaultLanguage(self):
         """ returns the current default language (e.g. zh_CN) """
-        for line in open(self.environment).readlines():
-            line = line.strip()
-            if line.startswith("LANGUAGE="):
-                (key,value) = line.split("=")
-                value = value.strip('"')
-                return value.split(":")[0]
-        for line in open(self.environment).readlines():
-            match = re.match(r'LANG="([a-zA-Z_]*).*"$',line)
-            if match:
-                return match.group(1)
+        for environment in self.environments:
+            if not os.path.exists(environment):
+                continue
+            for line in open(environment).readlines():
+                line = line.strip()
+                if line.startswith("LANGUAGE="):
+                    (key,value) = line.split("=")
+                    value = value.strip('"')
+                    return value.split(":")[0]
+            for line in open(environment).readlines():
+                match = re.match(r'LANG="([a-zA-Z_]*).*"$',line)
+                if match:
+                    return match.group(1)
+        return None
 
 if __name__ == "__main__":
     datadir = "/usr/share/language-selector/"
-    li = LocaleInfo("%s/data/languages" % datadir,
-                    "%s/data/countries" % datadir,
-                    "%s/data/languagelist" % datadir)
+    li = LocaleInfo("%s/data/languagelist" % datadir)
 
     print "default: '%s'" % li.getDefaultLanguage()
 
