@@ -8,7 +8,6 @@ import pygtk
 pygtk.require('2.0')
 import gtk
 import gtk.gdk
-import gtk.glade
 import pango
 import gobject
 import os.path
@@ -39,9 +38,10 @@ from gettext import gettext as _
 (COMBO_LANGUAGE,
  COMBO_CODE) = range(2)
 
+(IM_CHOICE,
+ IM_NAME) = range(2)
 
-
-from LanguageSelector.gtk.SimpleGladeApp import SimpleGladeApp
+from LanguageSelector.gtk.SimpleGtkbuilderApp import SimpleGtkbuilderApp
 from LanguageSelector.LocaleInfo import LocaleInfo
 from LanguageSelector.LanguageSelector import *
 from LanguageSelector.ImSwitch import ImSwitch
@@ -126,13 +126,12 @@ class GtkProgress(apt.OpProgress):
     def hide(self):
         self._window.hide()
 
-class GtkLanguageSelector(LanguageSelectorBase,  SimpleGladeApp):
+class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
 
     def __init__(self, datadir, options):
         LanguageSelectorBase.__init__(self, datadir)
-        SimpleGladeApp.__init__(self,
-                                datadir+"/data/LanguageSelector.glade",
-                                domain="language-selector")
+        SimpleGtkbuilderApp.__init__(self,
+                                datadir+"/data/LanguageSelector.ui")
 
         self.is_admin = grp.getgrnam("admin")[2] in os.getgroups()
         # see if we have any other human users on this system
@@ -161,8 +160,15 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGladeApp):
         combo.add_attribute(cell, 'text', COMBO_LANGUAGE)
         combo.set_model(model)
         self.combo_userlang_dirty = False
-        self.imSwitch = ImSwitch()
         self.options = options
+
+        combo = self.combobox_input_method
+        model = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
+        cell = gtk.CellRendererText()
+        combo.pack_start(cell, True)
+        combo.add_attribute(cell, 'text', IM_NAME)
+        combo.set_model(model)
+        self.imSwitch = ImSwitch()
         self._blockSignals = False
 
         # build the treeview
@@ -206,7 +212,7 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGladeApp):
             self.verifyInstalledLangPacks()
 
         if not self.imSwitch.available():
-            self.checkbutton_enable_input_methods.set_sensitive(False)
+            self.combobox_input_method.set_sensitive(False)
         self.setSensitive(True)
 
     def setSensitive(self, value):
@@ -266,11 +272,12 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGladeApp):
             
         def lang_view_func(cell_layout, renderer, model, iter):
             langInfo = model.get_value(iter, LIST_LANG_INFO)
+            langName = model.get_value(iter, LIST_LANG)
             inconsistent = langInfo.inconsistent
             if (langInfo.changes) :
-                markup = "<b>%s</b>" % langInfo.language
+                markup = "<b>%s</b>" % langName
             else:
-                markup = "%s" % langInfo.language
+                markup = "%s" % langName
             renderer.set_property("markup", markup)
 
         renderer = gtk.CellRendererText()
@@ -318,27 +325,13 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGladeApp):
         self.check_status()
         self.treeview_languages.queue_draw()
         #self.debug_pkg_status()
-    def on_checkbutton_basic_translations_clicked(self, button):
+    def on_checkbutton_translations_clicked(self, button):
         if self.block_toggle: return
         langInfo = self._get_langinfo_on_cursor()
         langInfo.languagePkgList["languagePack"].doChange = not langInfo.languagePkgList["languagePack"].doChange
         self.check_status()
         self.treeview_languages.queue_draw()
         #self.debug_pkg_status()
-    def on_checkbutton_extra_translations_clicked(self, button):
-        if self.block_toggle: return
-        langInfo = self._get_langinfo_on_cursor()
-        langInfo.languagePkgList["languageSupportTranslations"].doChange = not langInfo.languagePkgList["languageSupportTranslations"].doChange
-        self.check_status()
-        self.treeview_languages.queue_draw()
-        #self.debug_pkg_status()
-    def on_checkbutton_extra_clicked(self, button):
-        if self.block_toggle: return
-        langInfo = self._get_langinfo_on_cursor()
-        langInfo.languagePkgList["languageSupportExtra"].doChange = not langInfo.languagePkgList["languageSupportExtra"].doChange
-        self.check_status()
-        self.treeview_languages.queue_draw()
-        #self.debug_pkg_status()
 
     # the global toggle
     def on_toggled(self, renderer, path_string):
@@ -374,11 +367,9 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGladeApp):
         #print "on_treeview_languages_cursor_changed()"
         langInfo = self._get_langinfo_on_cursor()
         for (button, attr) in ( 
-              ("checkbutton_basic_translations", langInfo.languagePkgList["languagePack"]),
-              ("checkbutton_extra_translations", langInfo.languagePkgList["languageSupportTranslations"]),
+              ("checkbutton_translations", langInfo.languagePkgList["languagePack"]),
               ("checkbutton_writing_aids", langInfo.languagePkgList["languageSupportWritingAids"]),
               ("checkbutton_input_methods", langInfo.languagePkgList["languageSupportInputMethods"]),
-              ("checkbutton_extra", langInfo.languagePkgList["languageSupportExtra"]),
               ("checkbutton_fonts", langInfo.languagePkgList["languageSupportFonts"])  ):
             self.block_toggle = True
             if ((attr.installed and not attr.doChange) or (not attr.installed and attr.doChange)) :
@@ -411,9 +402,11 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGladeApp):
                             countRemove = countRemove + 1
                         else:
                             countInstall = countInstall + 1
-        #print "%d to install, %d to remove" % (countInstall, countRemove)
-        textInstall = gettext.ngettext("%d to install", "%d to install", countInstall) % countInstall
-        textRemove = gettext.ngettext("%d to remove", "%d to remove", countRemove) % countRemove
+        #print "%(INSTALL)d to install, %(REMOVE)d to remove" % (countInstall, countRemove)
+        # Translators: %(INSTALL)d is parsed; either keep it exactly as is or remove it entirely, but don't translate "INSTALL".
+        textInstall = gettext.ngettext("%(INSTALL)d to install", "%(INSTALL)d to install", countInstall) % {'INSTALL': countInstall}
+        # Translators: %(REMOVE)d is parsed; either keep it exactly as is or remove it entirely, but don't translate "REMOVE".
+        textRemove = gettext.ngettext("%(REMOVE)d to remove", "%(REMOVE)d to remove", countRemove) % {'REMOVE': countRemove}
         if countRemove == 0 and countInstall == 0: 
             self.label_install_remove.set_text("")
         elif countRemove == 0: 
@@ -468,37 +461,75 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGladeApp):
         if combo.get_active() < 0:
             return
         (lang, code) = model[combo.get_active()]
-        # check if that has a im-switch config
-        active = self.imSwitch.enabledForLocale(code)
-        self.checkbutton_enable_input_methods.set_active(active)
+        #print "Active language: "+code
 
-    def writeInputMethodConfig(self):
-        """ 
-        write new input method defaults - currently we only support all_ALL
-        """
+        combo = self.combobox_input_method
+        #cell = combo.get_child().get_cell_renderers()[0]
+        # FIXME: use something else than a hardcoded value here
+        #cell.set_property("wrap-width",300)
+        #cell.set_property("wrap-mode",pango.WRAP_WORD)
+        model = combo.get_model()
+        model.clear()
+
+        # find the default
+        currentIM = self.imSwitch.getInputMethodForLocale(code)
+        if currentIM == None:
+            currentIM = 'none'
+        #print "Current IM: "+currentIM
+
+        # find out about the other options
+        for (i, IM) in enumerate(self.imSwitch.getAvailableInputMethods()):
+            iter = model.append()
+            model.set(iter,
+                      IM_CHOICE,IM,
+                      IM_NAME, IM)
+            if IM == currentIM:
+                combo.set_active(i)
+        self.check_status()
+
+#    def writeInputMethodConfig(self):
+#        """ 
+#        write new input method defaults - currently we only support all_ALL
+#        """
+#        combo = self.combobox_user_language
+#        model = combo.get_model()
+#        if combo.get_active() < 0:
+#            return
+#        (lang, code) = model[combo.get_active()]
+#        # check if we need to do something
+#        new_value = self.checkbutton_enable_input_methods.get_active()
+#        if self.imSwitch.enabledForLocale(code) != new_value:
+#            if new_value:
+#                self.imSwitch.enable(code)
+#            else:
+#                self.imSwitch.disable(code)
+#            #self.showRebootRequired()
+#            #self.checkReloginNotification()
+
+#    @honorBlockedSignals
+#    def on_checkbutton_enable_input_methods_toggled(self, widget):
+#        #print "on_checkbutton_enable_input_methods_toggled()"
+#        active = self.checkbutton_enable_input_methods.get_active()
+#        self.combo_userlang_dirty = True
+#        self.setSensitive(False)
+#        self.writeInputMethodConfig()
+#        self.setSensitive(True)
+
+    @honorBlockedSignals
+    def on_combobox_input_method_changed(self, widget):
         combo = self.combobox_user_language
         model = combo.get_model()
         if combo.get_active() < 0:
             return
         (lang, code) = model[combo.get_active()]
-        # check if we need to do something
-        new_value = self.checkbutton_enable_input_methods.get_active()
-        if self.imSwitch.enabledForLocale(code) != new_value:
-            if new_value:
-                self.imSwitch.enable(code)
-            else:
-                self.imSwitch.disable(code)
-            #self.showRebootRequired()
-            #self.checkReloginNotification()
 
-    @honorBlockedSignals
-    def on_checkbutton_enable_input_methods_toggled(self, widget):
-        #print "on_checkbutton_enable_input_methods_toggled()"
-        active = self.checkbutton_enable_input_methods.get_active()
-        self.combo_userlang_dirty = True
-        self.setSensitive(False)
-        self.writeInputMethodConfig()
-        self.setSensitive(True)
+        combo = self.combobox_input_method
+        model = combo.get_model()
+        if combo.get_active() < 0:
+            return
+        (IM_choice, IM_name) = model[combo.get_active()]
+        #print "IM: "+IM_choice+"\t"+code
+        self.imSwitch.setInputMethodForLocale(IM_choice, code)
 
     @honorBlockedSignals
     def on_checkbutton_sync_languages_toggled(self, widget):
@@ -520,10 +551,10 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGladeApp):
         res = True
         try:
             for pkg in inst_list:
-                if self._cache.has_key(pkg):
+                if pkg in self._cache:
                     self._cache[pkg].markInstall()
             for pkg in rm_list:
-                if self._cache.has_key(pkg):
+                if pkg in self._cache:
                     self._cache[pkg].markDelete()
         except SystemError:
             res = False
@@ -540,8 +571,7 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGladeApp):
                 _("Could not install the selected language support"),
                 _("This is perhaps a bug of this application. Please "
                   "file a bug report at "
-                  "https://launchpad.net/bugs/bugs/+package/ against "
-                  "the 'language-selector' product.")))
+                  "https://bugs.launchpad.net/ubuntu/+source/language-selector/+filebug")))
             d.set_title=("")
             res = d.run()
             d.destroy()
@@ -572,7 +602,7 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGladeApp):
                 _("Could not install the full language support"),
                 _("Usually this is related to an error in your "
                   "software archive or software manager. Check your "
-                  "software preferences in the menu \"Adminstration\".")))
+                  "software preferences in the System > Administration menu.")))
             d.set_title("")
             d.run()
             d.destroy()
@@ -675,6 +705,9 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGladeApp):
         else:
             gtk.main_quit()
 
+    def hide_on_delete(self, widget, event):
+        return gtk.Widget.hide_on_delete(widget)
+
     def verifyInstalledLangPacks(self):
         """ called at the start to inform about possible missing
             langpacks (e.g. gnome/kde langpack transition)
@@ -750,19 +783,18 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGladeApp):
         #print "ll size: ", len(languageList)
         #print "ll type: ", type(languageList)
         for lang in languageList:
-            #print "langInfo: %s" % lang
+            #print "langInfo: %s (%s)" % (lang.language, lang.languageCode)
             inconsistent = lang.inconsistent
             #if inconsistent:
             #    print "inconsistent", lang.language
             installed = lang.fullInstalled
-            self._langlist.append([_(lang.language), lang])
+            lang_name = self._localeinfo.translate(lang.languageCode)
+            self._langlist.append([lang_name, lang])
         self._langlist.set_sort_column_id(LIST_LANG, gtk.SORT_ASCENDING)
         for button in ( 
-              "checkbutton_basic_translations",
-              "checkbutton_extra_translations",
+              "checkbutton_translations",
               "checkbutton_writing_aids",
               "checkbutton_input_methods",
-              "checkbutton_extra",
               "checkbutton_fonts"):
             self.block_toggle = True
             getattr(self, button).set_active(False)
